@ ------------------------------ LP.g ------------------------------------------

    Author:  Alfred Maussner
    Purpose: Solve and simulate the limited participation model. The model is 
             a more general version of the model in Christiano, Eichbaum,
             and Evans  (1997). It allows capital and labor augmenting technical
             progress. The model is calibrated with respect to (West) German
             economic data.

   Date:     25.08.2004
  
------------------------------------------------------------------------------- @

/* clear memory */
new;

library DGE, pgraph;

/* parameters and output options */
a=1.005;
alpha=0.73;
beta=0.994;
eta=2.0;
delta=0.011;
nstar=0.13;
nu=5.0;

rhoz=0.90;
sigmaz=0.0072;
rhomu=0.0;
sigmamu=0.0173;
mustar=1.0167;

lauf=0;

HP=1; @ Do not apply the HP filter if HP=0 @
outfile="d:\\TBK\\TB5\\g\\Ch3\\LM.txt";
HeadLine="Results of simulations of the limited participation model";

/* compute steady state values and related parameters and elasticities */
   yk=((a^eta)-beta*(1-delta))/(beta*(1-alpha));
   ck=yk-(a+delta-1);
   yc=yk/ck;
   yi=yk/(a+delta-1);
qstar=mustar*(a^(eta-1))*(1/beta);
  xsi=zeros(3,1);
  xsi[1]=eta/(1-(alpha/nu)*yc*(1/qstar));
  xsi[2]=(alpha*eta)/(qstar*(1/yc)-(alpha/nu));
  xsi[3]=1-(beta*(a^(-eta)))*(1-delta);
nochmal:

/* set up the matrices from which SolveBK computes the coefficients of the policy functions */
{Cu,Cxl,Cz,Dxl,Fxl,Du,Fu,Dz,Fz,Rho}=Makemat;

@goto ende;@


/* Compute policy functions */
{Lxx,Lxz,Llx,Llz,Lux,Luz}=SolveLA(Cu,Cxl,Cz,Dxl,Fxl,Du,Fu,Dz,Fz,rho,3);

@goto ende;@
@goto RunModel;@

/* Compute impulse responses */
shock=2;
nobs=10;
{yt,kt,ct,it,nt,pit,rt,mt,wt}=Impulse(nobs,shock);

if not lauf;
  y1=yt;
  c1=ct;
  i1=it;
  n1=nt;  
  w1=wt;
 pi1=pit;
  r1=rt;
 m1=mt;

if shock==2; rhoMu=0.50;  endif;
lauf=1;
goto nochmal;
else;
 yt=y1~yt; 
  ct=c1~ct;
  it=i1~it;
  nt=n1~nt;
  wt=w1~wt;
 pit=pi1~pit;
  mt=m1~mt; 
 rt=r1~rt;
endif;

GraphSettings;
_pltype={6,3};
_plwidth=10;
_paxht=0.25;
_ptitlht=0.4;

_pline=(1~6~0~0~nobs~0~1~0~2);
t=seqa(1,1,nobs);
_ptek="LM_1.tkf";
_pcolor={0,0};
fonts("Simplex Simgrma");
xlabel("Quarter");
@ylabel("Percentage Deviation from Trend");@
ylabel("Perctg. Dev. from Trend");
xtics(1,nobs,1,0);
_plegstr="\202\114\202[\202\m\201]=0\000\202\114\202[\202\m\201]=0.5";
begwind;
window(4,2,1);
title("Output");
_plegctl=(1~9~t[6]~minc(yt[.,2]*100));
xy(t,yt*100);
nextwind;
title("Consumption");
_plegctl=(1~9~t[6]~minc(ct[.,2]*100));
xy(t,ct*100);
nextwind;
title("Investment");
_plegctl=(1~9~t[6]~0.5*maxc(it[.,2]*100));
xy(t,it*100);
nextwind;
title("Hours");
_plegctl=(1~9~t[6]~minc(nt[.,2]*100));
xy(t,nt*100);
nextwind;
title("Real Wage");
_plegctl=(1~9~t[6]~0.9*minc(wt[.,2]*100));
xy(t,wt*100);
nextwind;
title("Inflation");
_plegctl=(1~9~t[5]~0.5*maxc(pit[.,1]*100));
xy(t,pit*100);
nextwind;
title("Real EOP Money Balances.");
_plegctl=(1~9~t[6]~minc(mt[.,2]*100));
xy(t,mt*100);
nextwind;
title("Rate of Interest");
_plegctl=(1~9~t[7]~0.2*maxc(rt[.,2]*100));
xy(t,rt*100);
endwind;

goto ende;

RunModel:

/* Compute second moments from simulated time series */
nofs=300;
nobs=80;
{sx,sxy,rxy,rx}=RBCRUN(nobs,nofs);

/* Output of results */
PrintResults;


ende:
end;

/* --------------------------------------- Subroutines ---------------------- */


/* MAKEMAT: sets up the matrices Cu, etc., where:
**
** ut:=[chut,Nhut,pihut,gammahut]
**
** xt:=[khut,mhut,xhut,lambdahut,qhut] */

proc(10)=Makemat;

   local Cu,Cxl,Cz,Dxl,Fxl,Du,Fu,Dz,Fz;
   
   Cu=zeros(4,4);
   Cu[1,1]=-xsi[1];   Cu[1,2]=xsi[2];  Cu[1,4]=(1-qstar)/qstar;
   Cu[2,2]=alpha-nu;             
   Cu[3,1]=1;         Cu[3,3]=1;
   Cu[4,2]=alpha;     Cu[4,3]=1;                

   Cxl=zeros(4,5);
   Cxl[1,4]=1/qstar;
   Cxl[2,1]=alpha-1;  Cxl[2,5]=1;
   Cxl[3,2]=1;
   Cxl[4,1]=alpha-1;  Cxl[4,2]=(qstar/alpha)*(1/yc);  Cxl[4,3]=1-Cxl[4,2]; Cxl[4,5]=1;

   Cz=zeros(4,2);
   Cz[2,1]=-1;
   Cz[3,2]=1;
   Cz[4,1]=-1;  Cz[4,2]=(qstar/alpha)*(1/yc);

   Dxl=zeros(5,5);
   Dxl[1,1]=a;
   Dxl[2,1]=alpha*xsi[3]; Dxl[2,4]=-1;
   Dxl[3,4]=1/qstar;
   Dxl[4,4]=1;            Dxl[4,5]=1;
   Dxl[5,2]=1;
   
   Fxl=zeros(5,5);
   Fxl[1,1]=-(a^eta)/beta;
   Fxl[2,4]=1;
   Fxl[3,4]=-1;
   Fxl[4,4]=-1;
   Fxl[5,2]=-1;
   
   Du=zeros(5,4);
   Du[2,2]=alpha*xsi[3];
   Du[3,3]=1;            Du[3,4]=(1-qstar)/qstar;
   Du[4,3]=1;
  
   Fu=zeros(5,4);
   Fu[1,1]=-ck; Fu[1,2]=alpha*yk;
   Fu[5,3]=-1;
   
   Dz=zeros(5,2);
   Dz[2,1]=xsi[3];

   Fz=zeros(5,2);
   Fz[1,1]=yk;
   Fz[5,2]=1;
   
   Rho=zeros(2,2);
   Rho[1,1]=RhoZ;
   Rho[2,2]=RhoMu;
   
 retp(Cu,Cxl,Cz,Dxl,Fxl,Du,Fu,Dz,Fz,Rho);

endp;


/* IMPULSE computes and returns impulse responses */

proc(9)=IMPULSE(nobs,shock);

  local yt, wt,it,ut,t,zt,xt,sigma,lmt;  

  ut=zeros(4,nobs);
  yt=zeros(nobs,1);
  lmt=zeros(2,nobs);
  it=yt;
  wt=yt;
  xt=zeros(3,nobs+1);
  zt=zeros(2,nobs+1);

  sigma=sigmaZ|sigmaMu;

  /* Compute respones to shock */
  zt[shock,3]=sigma[shock]; @ Shock occurs in period three with size of Sigma percent @
  t=3;
   
   do until t>nobs;
      zt[.,t+1]=rho*zt[.,t];
      xt[.,t+1]=lxx*xt[.,t]+lxz*zt[.,t];
      ut[.,t]=lux*xt[.,t]+luz*zt[.,t];
      lmt[.,t]=llx*xt[.,t]+llz*zt[.,t];
      yt[t]=alpha*ut[2,t]+(1-alpha)*xt[1,t]+zt[1,t];
      it[t]=yi*yt[t]-(yi-1)*ut[1,t];
    wt[t]  =(nu-1)*ut[2,t];
      t=t+1;
   endo;
  
  retp(yt,xt[1,1:nobs]',ut[1,.]',it,ut[2,.]',ut[3,.]',lmt[2,.]',xt[2,1:nobs]',wt);
      @yt,  kt         , ct     ,it,  nt    ,dp/p    , Rt     , mt+1        ,wt@
endp;

proc(4)=RBCRun(nobs,nofs);

   local xt, zt, ct, nt, ut, yt, it, st, wt, itn, epszt, epsmut, pit, rt, mt, lmt,
         sx, sxy, rxy, rx, mom1,mom2, nvars;

   nvars=8;   
   sx=zeros(nofs,nvars);
   sxy=zeros(nofs,nvars-1);
   rxy=zeros(nofs,nvars-1);
   rx=zeros(nofs,nvars);

  /* Simulation of time series */

  itn=1;cls; locate 5,5;?"Simulation No. ";
  do until itn>nofs;
    locate 6,5;ftos(itn,"*.*lf",4,0);pause(1/1000);

    /* Initialize vectors */

       xt=zeros(3,nobs+1);
       zt=zeros(2,nobs+1);
       ut=zeros(4,nobs);
       lmt=zeros(2,nobs);
    epszt= sigmaz*rndn(nobs+1,1);
   epsmut=sigmamu*rndn(nobs+1,1);
  zt[1,1]=epszt[1];  
  zt[2,1]=epsmut[1];
       ct=zeros(nobs,1);
       yt=ct;
       nt=ct;
       it=ct;
       wt=ct;
      pit=ct;
       rt=ct;

  /* compute time series */
  t=1;
  do until t>nobs;
    zt[.,t+1]=rho*zt[.,t]+(epszt[t+1]|epsmut[t+1]);
    xt[.,t+1]=Lxx*xt[.,t]+Lxz*zt[.,t];
    ut[.,t]  =Lux*xt[.,t]+Luz*zt[.,t];
    lmt[.,t]  =Llx*xt[.,t]+Llz*zt[.,t];
    yt[t]  =alpha*ut[2,t]+(1-alpha)*xt[1,t]+zt[1,t];
    wt[t]  =(nu-1)*ut[2,t];
    it[t]  =yi*yt[t]-(yi-1)*ut[1,t];
      t=t+1;
  endo;
  /*locate 7,7;
  **?maxc(xt[3,1:nobs]');
  **locate 8,1;
  **minc(xt[3,1:nobs]');?"Taste";wait;*/

  /* Filtering */
  if HP;
    {st,ct}=HPTREND(ut[1,.]',1600);
    {st,nt}=HPTREND(ut[2,.]',1600);
    {st,yt}=HPTREND(yt,1600);
    {st,it}=HPTREND(it,1600);
    {st,wt}=HPTREND(wt,1600);
   {st,pit}=HPTREND(ut[3,.]',1600);
    {st,rt}=HPTREND(lmt[2,.]',1600);
    {st,mt}=HPTREND(xt[2,1:nobs]',1600);
  else;
    ct=ut[1,.]';
    nt=ut[2,.]';
   pit=ut[3,.]';
    rt=lmt[2,.]';
    mt=xt[2,1:nobs]';
  endif;
  st=(yt~ct~it~nt~wt~pit~rt~mt);
  mom1=vcx(st);
  mom2=corrx(trimr(st,1,0)~trimr(st,0,1));  
  sx[itn,.]=sqrt(diag(mom1))';
  sxy[itn,.]=sx[itn,2:nvars]./sx[itn,1];
  rxy[itn,.]=mom2[1,2:nvars];
  rx[itn,.]=diag(mom2[1:nvars,nvars+1:2*nvars])';

  itn=itn+1;

endo;

retp(sx*100,sxy,rxy,rx);

endp;

/* PrintResults: prints second moments to a file */

proc(0)=PrintResults;

   local out, i, var, j;

   output file=^outfile reset;
   ?Headline;
   datestr(0) "   " timestr(0);
   ?"";
   ?"Parameter:";
   ?"a      = " ftos(a,      "*.*lf",8,4);
   ?"alpha  = " ftos(alpha,  "*.*lf",8,4);
   ?"beta   = " ftos(beta,   "*.*lf",8,4);
   ?"eta    = " ftos(eta,    "*.*lf",8,4);
   ?"nu     = " ftos(nu,     "*.*lf",8,4);
   ?"delta  = " ftos(delta,  "*.*lf",8,4);
   ?"rhoZ   = " ftos(rhoZ,   "*.*lf",8,4);
   ?"sigmaZ = " ftos(sigmaz, "*.*lf",8,4);
   ?"mustar = " ftos(mustar, "*.*lf",8,4);
   ?"RhoMu  = " ftos(RhoMu,  "*.*lf",8,4);
   ?"sigmaMu= " ftos(SigmaMu,"*.*lf",8,4);
   ?"";
   ?"Number of Simulations: " ftos(nofs,"*.*lf",4,0);
   ?"Length of time series: " ftos(nobs,"*.*lf",3,0);
   ?"";
   var="Output  "~"        "|
       "Consumpt"~"ion     "|
       "Investme"~"nt      "|
       "Hours   "~"        "|
       "Real Wag"~"e       "|
       "Inflatio"~"n Rate  "|
       "Rate of "~"Interest"|
       "Real Mon"~"ey Bala.";

   Outhlp1(var[1,.],meanc(sx[.,1]),1,meanc(rx[.,1]),1);
   i=zeros(2,2);
   i[1,.]=GetInt( sx[.,1],0.05);
   i[2,.]=GetInt( rx[.,1],0.05);
   Outhlp2(i[1,.],(0~0),i[2,.],(0~0));

   j=2;
   do until j>rows(var);

     Outhlp1(var[j,.],meanc(sx[.,j]),meanc(sxy[.,j-1]),meanc(rx[.,j]),meanc(rxy[.,j-1]));
     i=zeros(4,2); 
     i[1,.]=GetInt( sx[.,j]  ,0.05);
     i[2,.]=GetInt(sxy[.,j-1],0.05);
     i[3,.]=GetInt(rxy[.,j-1],0.05);
     i[4,.]=GetInt( rx[.,j]  ,0.05);
     Outhlp2(i[1,.],i[2,.],i[4,.],i[3,.]);
     j=j+1;

  endo;
  ?"";?"";
  ?"Column 1: Variable name";
  ?"Column 2: Standard deviation";
  ?"Column 3: Standard deviation in relation to standard deviation of output";
  ?"Column 4: First order autocorrelation";
  ?"Column 5: Crosscorrelation with output";


output off;
   
output off;
retp;
endp;
  
proc(0)=Outhlp1(s,n1,n2,n3,n4);

  local out;
  out=strput(s[2],s[1],9)
      $+"    "
      $+ftos(n1,"*.*lf",4,2)
      $+"           "
      $+ftos(n2,"*.*lf",4,2)
      $+"           "
      $+ftos(n3,"*.*lf",4,2)
      $+"           "
      $+ftos(n4,"*.*lf",4,2);
  $out;
retp;
endp;

proc(0)=Outhlp2(i1,i2,i3,i4);

  local out;
  
  out="                " $+ ftos(i1[1],"(*.*lf",4,2) $+"-" $+ftos(i1[2],"*.*lf)",4,2)$+"    "
                         $+ ftos(i2[1],"(*.*lf",4,2) $+"-" $+ftos(i2[2],"*.*lf)",4,2)$+"    "
                         $+ ftos(i3[1],"(*.*lf",4,2) $+"-" $+ftos(i3[2],"*.*lf)",4,2)$+"    "
                         $+ ftos(i4[1],"(*.*lf",4,2) $+"-" $+ftos(i4[2],"*.*lf)",4,2);
  $out;
retp;
endp;

/* GetInt: Sortiert den Vektor x in austeigender Reihenfolge und
**         gibt einen 1 mal 2 Vektor int zurueck, mit der Eigenschaft,
**         dass (p/2) Prozent der Werte in x unter int[1] und
**         1-(p/2)-Prozent der Werte in x ueber int[2] liegen.
*/

proc(1)=GetInt(x,p);

  local n, i, sx;
  n=rows(x);
  i=round(p*n/2);
 sx=sortc(x,1); 
  
retp(sx[i]~sx[n-i]);

endp;

/* GetLambda: Berechnet die Eigenwerte. */

proc(1)=GetLambda;

   local Cui, mbar, w, lambda, p, i;

   Cui=inv(Cu);
  mbar=inv(Dxl-Du*Cui*Cxl);
     w=-mbar*(Fxl-Fu*Cui*Cxl);
 
  {lambda,p}=eigv(w);
   i=sortind(abs(lambda));
   lambda=lambda[i];

retp(lambda);
endp;

proc(0)=TexOutput;
 local out, i,j, fmt1,fmt2,mask,moment2c;
load path=d:\lec\modelle\out;
load moment2c;
out="Produkti"~"on" |
    "Konsum  "~""   |
    "Investit"~"ionen"|
    "Arbeitss"~"tunden"|
    "Reallohn"~""|
    "Inflatio"~"nsrate"|
    "Nominalz"~"ins"|
    "Reale Ge"~"ldmenge";

out=out~meanc(sx)~(1.0|meanc(sxy))~(meanc(rx))~(1.0|meanc(rxy));

mask=0~0~1~1~1~1~0;

fmt1={"-*.*s" 8 8,"-*.*s&" 8 8,"*.*lf&"   6 2,"*.*lf&"   6 2,"*.*lf&"   6 2,"*.*lf"   6 2,"*.*s" 8 8};
fmt2={"-*.*s" 8 8,"-*.*s&" 8 8,"(*.*lf)&" 6 2,"(*.*lf)&" 6 2,"(*.*lf)&" 6 2,"(*.*lf)" 6 2,"*.*s" 8 8};

if SigmaMu==0;
  output file=d:\lec\modelle\TeX\Tab8bDaten.TeX reset;
else;
  output file=d:\lec\modelle\TeX\Tab8aDaten.TeX reset;
endif;
i=1;
do until i>rows(out);
   j=printfm(out[i,.]~"\\\\\l",mask,fmt1);
   j=printfm(" "~" "~moment2c[i,3:cols(moment2c)]~"\\\\\l",mask,fmt2);
   i=i+1;
endo;
output off;
retp;
endp;

@ -------------------------------- SolveBK ---------------------------------------------

   Zweck: Berechnen der Feed-Backregeln loglinearisierter Modelle mit rekursiver
          Struktur nach dem Verfahren von Blanchard und Kahn (1980).

   Aufruf: {xx,xz,lx,lz,ux,uz}=SolveBK(Cu,Cxl,Cz,Dxl,Fxl,Du,Fu,Dz,Fz,Rho,nx);

   Input:  Die Matrizen aus der Gleichung (I.8.1) im Vorlesungsskript, die Matrix Rho
           des AR(1)-Prozesses fuer die Zufallsschocks (GrossPhi im des Vorlesungsskript)
           und nx die Anzahl der Variablen im Vektor x.

   Output: Die Matrizen Lxx, Lxz, Llx, Llz, Lux und Luz (kleines l fuer Lambda)
           aus den Gleichungen (I.8.11), (I.8.12) und (I.8.15) des Skriptes.

   Bemerkung: Das Programm schreibt Zwischenergebnisse in die Datei SolveBK.out im
              jeweiligen Verzeichnis. Falls diese Ausgaben unterdrueck werden sollen,
              muessen die jeweiligen Stellen auskommentiert werden.

--------------------------------------------------------------------------------------- @

proc (6)=SolveBK(mu1,mxl1,mz1,mxl21,mxl22,mu21,mu22,mz21,mz22,rho,nx);

   local nu,nl,nz,xx,xz,lx,lz,ux,uz,mu1i,mbar,w,r,q,lambda,p,i,
         lambd1,lambd2,p11,p12,p21,p22,p11i,p12i,p21i,p22i,rx,rl,
         qx,ql,phi0,phi1,psi,p1,n,j,norm, add, lambd3, komplex, tol1;

   clear xx,xz,lx,lz,ux,uz, komplex;

    /* Toleranz fuer komplexe Wurzeln    */
    tol1=1e-10;

    /* Bestimmen der Dimensionen */

   nu=rows(mu1);nl=cols(mxl1)-nx;nz=cols(mz1);

/* Berechnen der Matrizen w, r und q in der Gleichung
**
** Et(x[t+1]|l[t+1]) = w*(x[t]|l[t]) + r*Et(z[t+1]) + q*z[t]
**
*/

mu1i=inv(mu1);
mbar=inv(mxl21-mu21*mu1i*mxl1);

w=-mbar*(mxl22-mu22*mu1i*mxl1);
r=mbar*(mz21+mu21*mu1i*mz1);
q=mbar*(mz22+mu22*mu1i*mz1);

/*  Bestimmen der Eigenwerte von w, Test auf relle Eigenwerte, Test
**  auf nx Eigenwerte im und nl Eigenwerte ausserhalb des Einheitskreises
*/

{lambda,p}=eigv(w);
i=sortind(abs(lambda));
lambda=lambda[i];

output file=SolveBK.out reset;
outwidth 200;
?"Eigenwerte:";
?lambda;

/* Pruefen, ob die Stabilitaetsvoraussetzungen erfuellt sind */
if abs(lambda[1:nx]) > 1;
  ?"Nicht alle der " ftos(nx,"%*.*lf",2,0)
   " Eigenwerte liegen im Einheitskreis";
  ?"Das Programm stoppt!";
  ?lambda;?"";
  ?"Weiter mit beliebiger Taste ...";wait;
  retp(xx,xz,lx,lz,ux,uz);
endif;

if sumc( abs(lambda[nx+1:rows(lambda)]) .< 1) > 0;
  ?"Nicht alle der " ftos(nl,"%*.*lf",2,0)
   " Eigenwerte liegen ausserhalb des Einheitskreises!";
  ?"Das Programm stoppt!";
  ?lambda;?"";
  ?" Weiter mit beliebiger Taste ...";wait;
  retp(xx,xz,lx,lz,ux,uz);

endif;

/* Aufstellen der Jordanform sowie der zugehoerigen Basis p */

p=p[.,i];
p1=zeros(rows(p),cols(p));
n=rows(lambda);
j=1;
do until j>n;

     if abs(sumc(imag(p[.,j])))>tol1;
        p1[.,j]=real(p[.,j]);
        norm=sqrt(sumc(real(p1[.,j]^2)));
        p1[.,j]=p1[.,j]./norm;
        p1[.,j+1]=imag(p[.,j]);
        p1[.,j+1]=p1[.,j+1]/norm;
        j=j+2;
     else;
        p1[.,j]=real(p[.,j]);
        norm=(sqrt(sumc(real(p1[.,j]^2))));
        p1[.,j]=p1[.,j]/norm;
        j=j+1;
     endif;

endo;

lambd1=zeros(nx,nx);
lambd2=zeros(nl,nl);

j=1;
do until j>nx;

   if abs(imag(lambda[j]))>1e-10;
      lambd1[j,j]=real(lambda[j]);
      lambd1[j+1,j+1]=lambd1[j,j];
      lambd1[j,j+1]=(imag(lambda[j]));
      lambd1[j+1,j]=-(imag(lambda[j]));
      j=j+2;
   else;
      lambd1[j,j]=real(lambda[j]);
      j=j+1;
   endif;

endo;

i=1;
do until j>n;

   if abs(imag(lambda[j]))>tol1;
      lambd2[i,i]=real(lambda[j]);
      lambd2[i+1,i+1]=lambd2[i,i];
      lambd2[i,i+1]=(imag(lambda[j]));
      lambd2[i+1,i]=-(imag(lambda[j]));
      komplex=1;
      i=i+2;j=j+2;
   else;
      lambd2[i,i]=real(lambda[j]);
      i=i+1;j=j+1;
   endif;

endo;

?lambd1;
?lambd2;

?inv(p1)*w*p1; @ Rechnerische Jordanform  zum Verlgeich @


/* Partitionieren von w, p, r und q */
p=p1;
p11=p[1:nx,1:nx];
p12=p[1:nx,nx+1:cols(p)];
p21=p[nx+1:rows(p),1:nx];
p22=p[nx+1:rows(p),nx+1:cols(p)];
p=inv(p);
p11i=p[1:nx,1:nx];
p12i=p[1:nx,nx+1:cols(p)];
p21i=p[nx+1:rows(p),1:nx];
p22i=p[nx+1:rows(p),nx+1:cols(p)];

rx=r[1:nx,.];
rl=r[nx+1:rows(r),.];
qx=q[1:nx,.];
ql=q[nx+1:rows(q),.];

/* Berechnen der Matrizen phi0, phi1 und psi */

phi0=p21i*rx+p22i*rl;
phi1=p21i*qx+p22i*ql;
psi=zeros(nl,nz);

if komplex;
   lambd3=inv(lambd2);
   add=lambd3*(phi0*rho+phi1);
   i=1;
   do until abs(add)<1e-20;
     psi=psi-add;
     add=lambd3*add*rho;
     i=i+1;
   endo;
   ?"Konvergenz der Summe nach " ftos(i,"*.*lf",6,0) " Runden ";
else;
   i=1;
   do until i>nl;
      psi[i,.]=-(lambd2[i,i]^(-1))*(phi0[i,.]*rho+phi1[i,.])*
               inv(eye(nz)-(lambd2[i,i]^(-1))*rho);
      i=i+1;
   endo;
endif;

/* Abschliessend werden die Matrizen aus den Loesungsformeln berechnet */

xx=p11*lambd1*inv(p11);
xz=(p11*lambd1*p12i+p12*lambd2*p22i)*inv(p22i)*psi+rx*rho+qx;

lx=-inv(p22i)*p21i;
lz=inv(p22i)*psi;

ux=mu1i*mxl1*(eye(nx)|lx);
uz=mu1i*mxl1*(zeros(nx,nz)|lz)+mu1i*mz1;

output off;

retp(xx,xz,lx,lz,ux,uz);

endp;

/* Das GAUSS-Unterprogramm HPTREND berechnet den HP-Filter. Input ist ein */
/* Vektor x und der Wert fuer MU. Output ist die gefilterte Reihe sowie   */
/* der resultierende Trend.                                               */



proc (2)=HPTREND(x,mu);

local m, i, nobs, a;

  nobs=rows(x);
  
  m=zeros(nobs,3);

  /* Belegen der Matrix */
  i=1;
  do until i>nobs;
   m[i,1]=mu;
   m[i,2]=-4*mu;
   m[i,3]=1+6*mu;
        i=i+1;
  endo;
  m[1,1]=0;         m[2,1]=0;
  m[1,2]=0;         m[2,2]=-2*mu;     m[nobs,2]=m[2,2];
  m[1,3]=1+mu;      m[2,3]=1+5*mu;
  m[nobs,3]=m[1,3]; m[nobs-1,3]=m[2,3];
    
  a=bandsolpd(x,m);     
  
 retp(a,x-a);

endp;

